var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
define(["require", "exports", "./heading-id-generator"], function (require, exports, heading_id_generator_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.toc = void 0;
    heading_id_generator_1 = __importDefault(heading_id_generator_1);
    function nPrefix(str, n) {
        var output = "";
        for (var i = 0; i < n; i++) {
            output += str;
        }
        return output;
    }
    /**
     *  eg:
     * Haha [A](www.a.com) xxx [B](www.b.com)
     *  should become
     * Haha A xxx B
     *
     * check issue #41
     */
    function sanitizeContent(content) {
        var output = "";
        var offset = 0;
        // test ![...](...)
        // test [...](...)
        // <a name="myAnchor"></a>Anchor Header
        // test [^footnote]
        var r = /\!?\[([^\]]*)\]\(([^)]*)\)|<([^>]*)>([^<]*)<\/([^>]*)>|\[\^([^\]]*)\]/g;
        var match = null;
        // tslint:disable-next-line:no-conditional-assignment
        while ((match = r.exec(content))) {
            output += content.slice(offset, match.index);
            offset = match.index + match[0].length;
            if (match[0][0] === "<") {
                output += match[4];
            }
            else if (match[0][0] === "[" && match[0][1] === "^") {
                //  # footnote
                output += "";
            }
            else if (match[0][0] !== "!") {
                output += match[1]; // image
            }
            else {
                output += match[0];
            }
        }
        output += content.slice(offset, content.length);
        return output;
    }
    /**
     *
     * @param opt:TocOption =
     * @param tokens = [{content:string, level:number, id:optional|string }]
     * @return {content, array}
     */
    function toc(tokens, opt) {
        var headingIdGenerator = new heading_id_generator_1.default();
        if (!tokens) {
            return { content: "", array: [] };
        }
        var ordered = opt.ordered;
        var depthFrom = opt.depthFrom || 1;
        var depthTo = opt.depthTo || 6;
        var tab = opt.tab || "  ";
        var ignoreLink = opt.ignoreLink || false;
        if (ordered) {
            tab = "    ";
        }
        tokens = tokens.filter(function (token) {
            return token.level >= depthFrom && token.level <= depthTo;
        });
        if (!tokens.length) {
            return { content: "", array: [] };
        }
        var outputArr = [];
        var smallestLevel = tokens[0].level;
        // get smallestLevel
        for (var _i = 0, tokens_1 = tokens; _i < tokens_1.length; _i++) {
            var token = tokens_1[_i];
            if (token.level < smallestLevel) {
                smallestLevel = token.level;
            }
        }
        var orderedListNums = [];
        for (var _a = 0, tokens_2 = tokens; _a < tokens_2.length; _a++) {
            var token = tokens_2[_a];
            var content = token.content.trim();
            var level = token.level;
            var slug = token.id || headingIdGenerator.generateId(content);
            var n = level - smallestLevel;
            var numStr = "1";
            if (ordered) {
                // number for ordered list
                if (n >= orderedListNums.length) {
                    orderedListNums.push(1);
                }
                else if (n === orderedListNums.length - 1) {
                    orderedListNums[orderedListNums.length - 1]++;
                }
                else {
                    orderedListNums = orderedListNums.slice(0, n + 1);
                    if (orderedListNums.length) {
                        orderedListNums[orderedListNums.length - 1]++;
                    }
                }
                numStr = orderedListNums[orderedListNums.length - 1];
            }
            var listItem = "" + nPrefix(tab, n) + (ordered ? numStr + "." : "-") + " " + (ignoreLink
                ? sanitizeContent(content)
                : "[" + sanitizeContent(content) + "](#" + slug + ")");
            outputArr.push(listItem);
        }
        return {
            content: outputArr.join("\n"),
            array: outputArr,
        };
    }
    exports.toc = toc;
});
//# sourceMappingURL=toc.js.map